/****************************************************************************
** $Id: rs_grid.cpp,v 1.6 2003/09/15 12:58:36 andrew Exp $
**
** Copyright (C) 2001-2003 RibbonSoft. All rights reserved.
**
** This file is part of the qcadlib Library project.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid qcadlib Professional Edition licenses may use 
** this file in accordance with the qcadlib Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.ribbonsoft.com for further details.
**
** Contact info@ribbonsoft.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "rs_grid.h"
#include "rs_units.h"
#include "rs_graphic.h"

/**
 * Constructor.
 */
RS_Grid::RS_Grid(RS_GraphicView* graphicView) {
    this->graphicView = graphicView;
    pt = NULL;
    number = 0;
    metaX = NULL;
    numMetaX = 0;
    metaY = NULL;
    numMetaY = 0;
}


/**
 * Destructor.
 */
RS_Grid::~RS_Grid() {
    if (pt!=NULL) {
        delete[] pt;
    }
    if (metaX!=NULL) {
        delete[] metaX;
    }
    if (metaY!=NULL) {
        delete[] metaY;
    }
}


/**
 * Updates the grid point array.
 */
void RS_Grid::update() {
    if (graphicView->isGridOn()) {
        // delete old grid:
        if (pt!=NULL) {
            delete[] pt;
            pt = NULL;
        }
        if (metaX!=NULL) {
            delete[] metaX;
            metaX = NULL;
        }
        if (metaY!=NULL) {
            delete[] metaY;
            metaY = NULL;
        }

        // find out unit:
        RS_Graphic* graphic = graphicView->getGraphic();
        RS2::Unit unit = RS2::None;
        if (graphic!=NULL) {
            unit = graphic->getUnit();
        }

        double gridWidthX;
        double gridWidthY;
        double metaGridWidthX;
        double metaGridWidthY;

        // init grid spacing:
        // metric grid:
        if (RS_Units::isMetric(unit) || unit==RS2::None) {
            gridWidthX = 0.000001;

            // auto scale grid
            while (graphicView->toGuiDX(gridWidthX)<10) {
                gridWidthX*=10;
            }
            metaGridWidthX = gridWidthX*10;
        }

        // imperial grid:
        else {
            gridWidthX = 1.0/1024.0;

            if (unit==RS2::Inch) {

                // auto scale grid
                while (graphicView->toGuiDX(gridWidthX)<10) {
                    if (RS_Math::round(gridWidthX)>=36) {
                        gridWidthX*=2;
                    } else if (RS_Math::round(gridWidthX)>=12) {
                        gridWidthX*=3;
                    } else if (RS_Math::round(gridWidthX)>=4) {
                        gridWidthX*=3;
                    } else if (RS_Math::round(gridWidthX)>=1) {
                        gridWidthX*=2;
                    } else {
                        gridWidthX*=2;
                    }
                }

                // metagrid shows inches..
                metaGridWidthX = 1.0;
				
                if (graphicView->toGuiDX(metaGridWidthX)<20) {
                	// .. or feet
                	metaGridWidthX = 12.0;

                	// .. or yards
                	if (graphicView->toGuiDX(metaGridWidthX)<20) {
                    	metaGridWidthX = 36.0;

                    	// .. or miles
                    	//if (graphicView->toGuiDX(metaGridWidthX)<20) {
                    	//    metaGridWidthX = 63360.0;
                    	//}

                    	// .. or nothing
                    	if (graphicView->toGuiDX(metaGridWidthX)<20) {
                       		metaGridWidthX = -1.0;
                    	}
					}
                }
            } else {
                while (graphicView->toGuiDX(gridWidthX)<10) {
                    gridWidthX*=2;
                }
                metaGridWidthX = -1.0;
            }
        }

        gridWidthY = gridWidthX;
        metaGridWidthY = metaGridWidthX;

        spacing = gridWidthX;
        metaSpacing = metaGridWidthX;

        if (gridWidthX>0.0) {
        	// find grid boundaries
            double left = (int)(graphicView->toGraphX(0) / 
					gridWidthX) * gridWidthX;
            double right = (int)(graphicView->toGraphX(graphicView->getWidth()) /
                                 gridWidthX) * gridWidthX;
            double top = (int)(graphicView->toGraphY(0) / 
					gridWidthY) * gridWidthY;
            double bottom = 
					(int)(graphicView->toGraphY(graphicView->getHeight()) /
                                  gridWidthY) * gridWidthY;

            left -= gridWidthX;
            right += gridWidthX;
            top += gridWidthY;
            bottom -= gridWidthY;
			
			
        	// calculate number of visible grid points
            int numberX = (RS_Math::round((right-left) / gridWidthX) + 1);
            int numberY = (RS_Math::round((top-bottom) / gridWidthY) + 1);
            number = numberX*numberY;

        	// create grid array:
            if (number>0 && number<10000) {
                pt = new RS_Vector[number];

                int i=0;
                for (int y=0; y<numberY; ++y) {
                    for (int x=0; x<numberX; ++x) {
                        pt[i++] = RS_Vector(left+x*gridWidthX, 
								bottom+y*gridWidthY);
                    }
                }
            } else {
                number = 0;
            }
        }

        // find meta grid boundaries
        if (metaGridWidthX>0.0) {
            double mleft = (int)(graphicView->toGraphX(0) /
                                 metaGridWidthX) * metaGridWidthX;
            double mright = (int)(graphicView->toGraphX(graphicView->getWidth()) /
                                  metaGridWidthX) * metaGridWidthX;
            double mtop = (int)(graphicView->toGraphY(0) /
                                metaGridWidthY) * metaGridWidthY;
            double mbottom = (int)(graphicView->toGraphY(graphicView->getHeight()) /
                                   metaGridWidthY) * metaGridWidthY;

            mleft -= metaGridWidthX;
            mright += metaGridWidthX;
            mtop += metaGridWidthY;
            mbottom -= metaGridWidthY;

        	// calculate number of visible meta grid lines:
            numMetaX = (RS_Math::round((mright-mleft) / metaGridWidthX) + 1);
            numMetaY = (RS_Math::round((mtop-mbottom) / metaGridWidthY) + 1);

        	// create meta grid arrays:
            metaX = new double[numMetaX];
            metaY = new double[numMetaY];

            int i=0;
            for (int x=0; x<numMetaX; ++x) {
                metaX[i++] = mleft+x*metaGridWidthX;
            }
            i=0;
            for (int y=0; y<numMetaY; ++y) {
                metaY[i++] = mbottom+y*metaGridWidthY;
            }
        }
    }
}


// EOF
